import { Component, OnInit } from '@angular/core';
import { ucapAnimations } from '@ucap-webmessenger/ui';
import { Observable } from 'rxjs';
import {
  DeptInfo,
  QueryProtocolService,
  DeptSearchType,
  UserInfoSS,
  DeptUserResponse
} from '@ucap-webmessenger/protocol-query';
import { Store, select } from '@ngrx/store';
import { NGXLogger } from 'ngx-logger';

import * as AppStore from '@app/store';
import * as QueryStore from '@app/store/messenger/query';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { LoginInfo, KEY_LOGIN_INFO } from '@app/types';
import { take, map, switchMap } from 'rxjs/operators';

@Component({
  selector: 'app-layout-chat-left-sidenav-organization',
  templateUrl: './organization.component.html',
  styleUrls: ['./organization.component.scss'],
  animations: ucapAnimations
})
export class OrganizationComponent implements OnInit {
  departmentInfoList$: Observable<DeptInfo[]>;
  selectedDepartmentUserInfoList$: Observable<UserInfoSS[]>;
  selectedDepartmentStatus$: Observable<DeptUserResponse>;

  constructor(
    private store: Store<any>,
    private queryProtocolService: QueryProtocolService,
    private sessionStorageService: SessionStorageService,
    private logger: NGXLogger
  ) {}

  ngOnInit() {
    this.departmentInfoList$ = this.store.pipe(
      select(AppStore.MessengerSelector.QuerySelector.departmentInfoList)
    );
    this.selectedDepartmentUserInfoList$ = this.store.pipe(
      select(
        AppStore.MessengerSelector.QuerySelector.selectedDepartmentUserInfoList
      )
    );
    this.selectedDepartmentStatus$ = this.store.pipe(
      select(AppStore.MessengerSelector.QuerySelector.selectedDepartmentStatus)
    );
  }

  onSelectedOrganization(deptInfo: DeptInfo) {
    const loginInfo = this.sessionStorageService.get<LoginInfo>(KEY_LOGIN_INFO);

    this.store
      .pipe(
        take(1),
        select(AppStore.AccountSelector.AuthenticationSelector.loginRes),
        map(loginRes => {
          this.store.dispatch(
            QueryStore.deptUser({
              divCd: 'ORG',
              companyCode: loginInfo.companyCode,
              seq: deptInfo.seq,
              search: '',
              searchRange: DeptSearchType.All,
              senderCompanyCode: loginInfo.companyCode,
              senderEmployeeType: loginRes.userInfo.employeeType
            })
          );
        })
      )
      .subscribe();
  }
}
